

/*
   ; Test the turret extension
   ; =========================

   ;------------------------------------------------------------------------
   ; Author:    Franzi Edo.     The 03/04/98
   ; Modifs:
   ;
   ; Project:   KHEPERA
   ; Goal:              This PRGM launches 2 processes.
   ;                    Process 0: "alive" it toggles the LED 0 every 500 ms.
   ;                    Process 1: it sends/receives a turret message every 10 ms.
   ;
   ; This module is (c) of Edoardo Franzi 1992-1998
   ; franzi@k-team.com
   ;
   ;------------------------------------------------------------------------
 */

#include	<sys/kos.h>
#include 	<stdlib.h>
#include 	<stdio.h>

#define		KIDTURRET	24	/* turret ID                      */
#define		KNBTRY		2	/* NB. of try to send the message */
#define		KNCPT		20	/* NB. of counter                 */

static uint32 vIDProcess[2];

/*
 * Send a counter vector to the turret
 * >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
 *
 * This function sends a counter vector to the turret.
 *
 * in:          counter vector
 * out:         0 OK
 *                      -1 for a network problem
 *
 */

int32
turret_putCounter (uint8 * counter)
{
  static uint8 send[KNCPT + 3] =
  {KIDTURRET, KNCPT + 1, 'P'};
  uint8 receive[1];
  int32 status;
  uint32 i;

  for (i = 0; i < KNCPT; i++)
    send[i + 3] = counter[i];
  RESERVE_MSG (KIDTURRET)
    status = msg_snd_rec_message (send, KNCPT + 3, receive, 1, KNBTRY);
  RELEASE_MSG (KIDTURRET)
    return status;
}

/*
 * Read a counter vector from the turret
 * >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
 *
 * This function reads a counter vector from the turret.
 *
 * in:          counter vector
 * out:         0 OK
 *                      -1 for a network problem
 *
 */

int32
turret_getCounter (uint8 * counter)
{
  static uint8 send[] =
  {KIDTURRET, 1, 'G'};
  uint8 receive[KNCPT + 1], i;
  int32 status;

  RESERVE_MSG (KIDTURRET)
    status = msg_snd_rec_message (send, 3, receive, KNCPT + 1, KNBTRY);
  RELEASE_MSG (KIDTURRET)
    for (i = 0; i < KNCPT; i++)
    counter[i] = receive[i + 1];
  return status;
}

/*
 * Communication problem (more than KNBTRY tries)
 * >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
 *
 * in:          -
 * out:         -
 *
 */

void
turret_communicationProblem (void)
{

  printf ("Communication problem\r\n");
}

/*
 * Process 0
 * ^^^^^^^^^
 *
 * This process changes the state of the LED 0
 * every 500 ms.
 *
 */

static void
process_0 ()
{

  for (;;)
    {
      tim_suspend_task (500);
      var_change_led (0);
    }
}

/*
 * Process 1
 * ^^^^^^^^^
 *
 * This process sends/receive a counter vector to/from the network
 * every 10 ms.
 *
 */

static void
process_1 ()
{
  uint8 sndCounter[KNCPT], recCounter[KNCPT];
  int32 status;
  uint32 message, i;
  boolean coherence;

  message = 0;
  for (i = 0; i < KNCPT; i++)
    {
      sndCounter[i] = 0;
      recCounter[i] = 0;
    }

  for (;;)
    {
      tim_suspend_task (10);

/* Send the counter ... */

      if (turret_putCounter (sndCounter) < 0)
	turret_communicationProblem ();
      for (i = 0; i < KNCPT; i++)
	sndCounter[i]++;

/* Read and check the counter ... */

      status = turret_getCounter (recCounter);
      if (status < 0)
	turret_communicationProblem ();
      else
	{
	  coherence = TRUE;
	  for (i = 0; i < KNCPT; i++)
	    {
	      if (sndCounter[i] != recCounter[i])
		coherence = FALSE;
	    }
	  if (coherence == FALSE)
	    {
	      printf ("Big problem ... The message is corrupted\r\n");
	    }
	}
      printf ("Message number = %ld\r", message);
      message++;
    }
}

/*
 * MAIN
 * ====
 *
 * - Initialise the used managers: tim & bios are initialised at the start-up.
 * - Launch all the processes.
 * - Kill the "main". At this moment only the launched processes are executed.
 *
 */

int
main (void)
{
  int32 status;

  /*
     --------------------------|------------------------------------------------|
   */

  static char prName_0[] = "User 0 process, EF-98                            Rev. 1.00\r\n";
  static char prName_1[] = "User 1 process, EF-98                            Rev. 1.00\r\n";

  com_reset ();
  var_reset ();

  status = install_task (prName_0, 800, process_0);
  if (status == -1)
    exit (0);
  vIDProcess[0] = (uint32) status;

  status = install_task (prName_1, 800, process_1);
  if (status == -1)
    exit (0);
  vIDProcess[1] = (uint32) status;

  return 0;
}
